/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _SYSTEM_H_
#define _SYSTEM_H_

#include <stdio.h>
#include <iostream>
#include "membership.h"
#include "rule.h"
#include "messages.h"

#define MIN_DEFAULT_LIMIT -100.0
#define MAX_DEFAULT_LIMIT 100.0

#if defined (MATLAB_MEX_FILE) // For compile in MATLAB
	#ifndef _MEX_H_
		#define _MEX_H_
		#include "mex.h"
	#endif
#endif

// Declarations ################################################################

class System
{
    int in,out,*N; // Inputs, outputs and y number of rules (for each output)
	double *min_in, *min_out,*max_in, *max_out; // Max and min limits for inputs and outputs of the system
    Rule **R; // Rules
public:
    int rules(int output) const; // Return the number of rules
    int inputs(void) const; // Return the number of inputs
    int outputs(void) const; // Return the number of outputs
	double in_min(int input) const; // Return the min limit of an input
	double in_max(int input) const; // Return the max limit of an input
	double out_min(int output) const; // Return the min limit of an output
	double out_max(int output) const; // Return the max limit of an output
	void in_min(double *limits); // Change the min limit of inputs
	void in_max(double *limits); // Change the max limit of inputs
	void out_min(double *limits); // Change the min limit of outputs
	void out_max(double *limits); // Change the max limit of outputs
    int test(void) const; // Returns 1 if all parameters of all membership functions of the system are valid
    int write_Rule(const Rule &R, int r, int output); // Write a rule. Return 0 if error
    int read_Rule(Rule &R, int r, int output) const; // Extract a rule. Return 0 if error
    Rule* read_Rule(int r, int output); // Extract a rule
    void initialize(int in, int out, int *N); // Initilice the system
    System & operator=(const System &S);
    System();
    System(const System &S);
    System(int in, int out, int *N);
    ~System();
};

#endif
