/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../base/fuzzy.h"
#include "aux_matlab.h"
#include "mex.h"

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    int i, NDC, n_con=2, out, rule;
    double *data;
    if(nrhs<1)
        ERRORMSG(E_NumberArgIn)
    if (nrhs>1 && nrhs!=3)
        ERRORMSG(E_NumberArgIn)
    if (nlhs>2)
        ERRORMSG(E_NumberArgOut)
    if (mxIsEmpty(prhs[0]) || mxIsNaN(*mxGetPr(prhs[0])))
        ERRORMSG(E_ArgNoValid)
    if (nrhs==3)
    {
        if (mxGetM(prhs[1])>1 || mxGetM(prhs[2])>1)
            ERRORMSG(E_File)
        if (mxGetN(prhs[1])!=mxGetN(prhs[2]))
            ERRORMSG(E_NumberArg)
    }
    System S;
    if (!readModel(prhs[0],S))
        ERRORMSG(E_Model)
    if (nrhs==1)
    {
        n_con=NumbersOfConsequentsData(S);
        if (n_con==0)
            ERRORMSG(E_BadModel)
    }
    else
    {
        n_con=0;
        for (i=0;i<mxGetN(prhs[1]);i++)
        {
            out=((int)*(mxGetPr(prhs[1])+i))-1; // MATLAB is index-1 but C/C++ is index-0
            rule=((int)*(mxGetPr(prhs[2])+i))-1;
            NDC=NumbersOfConsequentsData(S,out,rule);
            if (NDC==0)
                mexPrintf("??? Error using ==> conseq2mat\n%s, or\n%s\n",E_BadModel,E_NumberArgIn);
            n_con+=NDC;
        }
    }
    mxDestroyArray(plhs[0]);
    plhs[0]=mxCreateDoubleMatrix(1,n_con,mxREAL);
    data=mxGetPr(plhs[0]);
    if (nrhs==1)
        n_con=getConsequents(S,data);
    else
    {
        n_con=0;
        for (i=0;i<mxGetN(prhs[1]);i++)
        {
            out=((int)*(mxGetPr(prhs[1])+i))-1;
            rule=((int)*(mxGetPr(prhs[2])+i))-1;
            n_con+=getConsequents(S,data+NDC*i,out,rule);
        }
    }
    if (nlhs==2)
    {
        mxDestroyArray(plhs[1]);
        plhs[1]=mxCreateDoubleScalar(n_con);
    }
    if (n_con==0)
    {
        if (nlhs==1)
            ERRORMSG(E_Acquire)
        else
        {
            plhs[0]=mxCreateDoubleMatrix(0,0,mxREAL); // Empty matrix
            return;
        }
    }
}
