/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../base/fuzzy.h"
#include "aux_matlab.h"
#include "mex.h"

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
        int n,i,output,ode=0;
        
        if(nrhs<2 || nrhs>4)
            ERRORMSG(E_NumberArgIn)
        if (nlhs>1)
            ERRORMSG(E_NumberArgOut)
        for (i=0;i<nrhs;i++)
        {
            if (mxIsEmpty(prhs[i]) || mxIsNaN(*mxGetPr(prhs[i])))
                ERRORMSG(E_ArgNoValid)
        }
        double *X,*dX;
        System S,C;
        char error[MAX_FILE_NAME];

        if (nrhs!=2 && mxGetN(prhs[0])==1 && mxGetM(prhs[0])==1 && mxIsStruct(prhs[1])==false && mxIsChar(prhs[1])==false)
            ode=1; // If this function is called from 'ode', the 1st argument is the time

        // Read arguments
        X=mxGetPr(prhs[ode]);
        if (!X)
            ERRORMSG(E_NumberArgIn)
        if (mxGetN(prhs[ode])!=1)
            ERRORMSG(E_Column)
        if (!readModel(prhs[1+ode],S))
        {
            mexPrintf("%s %s.\n",U_CheckModel,O_OfPlant);
            ERRORMSG(E_Model)
        }
        n=S.outputs();
        plhs[0]=mxCreateDoubleMatrix(n,1,mxREAL);
        if (!plhs[0])
            ERRORMSG(E_NumberArgOut)
        dX=mxGetPr(plhs[0]);
        if (!dX)
        {
            mxDestroyArray(plhs[0]);
            ERRORMSG(E_NumberArgOut)
        }
        // Run
        if (nrhs==(2+ode))
        {// Evaluate the plant
            if (mxGetM(prhs[ode])!=S.inputs())
               ERRORMSG(E_PointCoherent)
            output=Evaluate(X, dX, S);
            if(output!=1)
            {
                mxDestroyArray(plhs[0]);
                mexPrintf("Fuzeval. %s or %s\n",U_Overflow,E_NoCoherent);
                return;
            }
        }
        else
        {// Evaluate the closed-loop system (plant + controller)
            if (mxGetM(prhs[ode])!=S.outputs())
                ERRORMSG(E_PointCoherent)
            if (!readModel(prhs[2+ode],C))
            {
                mxDestroyArray(plhs[0]);
                mexPrintf("%s %s.\n",U_CheckModel,O_OfController);
                ERRORMSG(E_Model)
            }
            output=Evaluate(X,dX,S,C);
            if (output!=1)
            {
                if (!error)
                {
                    mxDestroyArray(plhs[0]);
                    ERRORMSG(E_NoCoherent_BadX)
                }
                else
                {
                    mxDestroyArray(plhs[0]);
                    mexPrintf("Fuzeval. %s or",U_Overflow);
                    ERRORMSG(E_NoCoherent)
                }
            }
        }
}
