/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../base/fuzzy.h"
#include "aux_matlab.h"
#include "mex.h"

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    int i, n_ant, n_con, size_data, error,out,rule;
    double *data;
    if(nrhs<2)
        ERRORMSG(E_NumberArgIn)
    if (nrhs>2 && nrhs!=4)
        ERRORMSG(E_NumberArgIn)
    if (nlhs>2)
        ERRORMSG(E_NumberArgOut)
    if (mxIsEmpty(prhs[0]) || mxIsNaN(*mxGetPr(prhs[0])))
        ERRORMSG(E_ArgNoValid)
    System S;
    if (!readModel(prhs[0],S))
        ERRORMSG(E_Model)
    if (nrhs==2)
    {
        n_ant=NumbersOfAntecedentsData(S);
        n_con=NumbersOfConsequentsData(S);
        size_data=n_ant+n_con;
        if (size_data==0)
            ERRORMSG(E_BadModel)
    }
    else
    {
        out=((int)*mxGetPr(prhs[2]))-1; // MATLAB is index-1 but C/C++ is index-0
        rule=((int)*mxGetPr(prhs[3]))-1;
        n_ant=NumbersOfAntecedentsData(S,out,rule);
        n_con=NumbersOfConsequentsData(S,out,rule);
        size_data=n_ant+n_con;
        if (size_data==0)
            mexPrintf("??? Error using ==> mat2fuz\n%s, or\n%s\n",E_BadModel,E_NumberArgIn);
    }

    // Save data in S
    data=mxGetPr(prhs[1]);
    if (nrhs==2)
    {
        error=setAntecedents(S,data);
        error+=setConsequents(S,data+n_ant);
    }
    else
    {
        error=setAntecedents(S,data,out,rule);
        error&=setConsequents(S,data+n_ant,out,rule);
    }
    if (nlhs==2)
        plhs[1]=mxCreateDoubleScalar(error);
    if (error==0)
    {
        if (nlhs==1)
            ERRORMSG(E_Store)
        else
            plhs[0]=mxCreateDoubleMatrix(0,0,mxREAL); // Empty matrix
    }
    else
    {
        // Create the FIS variable
        const char* nombres[]={"name","type","andMethod","orMethod","defuzzMethod","impMethod","aggMethod","input","output","rule"};
        int dim[]={1,1};
        plhs[0]=mxCreateStructArray(2,dim,10,nombres);
        if(!System2FIS(S, plhs[0]))
        {
            mxDestroyArray(plhs[0]);
            ERRORMSG(E_CreateFIS)
        }
    }
}
